@@ -1,10 +1,29 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
+/* This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *
+ * (C) Copyright 2009 Novell, Inc.
+ */
+
 #include <arpa/inet.h>
+#include <glib/gi18n.h>
 #include <NetworkManager.h>
 #include <nm-gsm-device.h>
 #include <nm-cdma-device.h>
 #include <nm-connection.h>
+#include <nm-settings-interface.h>
 #include <nm-setting-connection.h>
 #include <nm-setting-serial.h>
 #include <nm-setting-ppp.h>
@@ -12,31 +31,40 @@
 #include <nm-setting-cdma.h>
 #include <nm-setting-ip4-config.h>
 #include <nm-utils.h>
+#include <nm-mobile-providers.h>
 #include "nmn-new-connection.h"
-#include "nma-gconf-settings.h"
-#include "nmn-wifi-list.h"
-#include "nmn-mobile-providers.h"
+#include "nm-list-model.h"
+#include "nm-wifi-item.h"
+#include "nm-gsm-pin-request-item.h"
+#include "nm-gconf-settings.h"
+#include "nmn-list.h"
 #include "wireless-dialog.h"
 
-G_DEFINE_TYPE (NmnNewConnection, nmn_new_connection, GTK_TYPE_EVENT_BOX)
+G_DEFINE_TYPE (NmnNewConnection, nmn_new_connection, GTK_TYPE_VBOX)
 
 enum {
     PROP_0,
-    PROP_BUILDER,
-    PROP_NM_DATA,
+    PROP_MODEL,
 
     LAST_PROP
 };
 
+enum {
+    CLOSE,
+
+    LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL];
+
 #define GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), NMN_TYPE_NEW_CONNECTION, NmnNewConnectionPrivate))
 
 typedef struct {
-    GtkBuilder *builder;
-    NmnNMData *nm_data;
+    NmnModel *model;
 
     /* WiFi widgets */
+    GtkWidget *wifi_list_container;
     GtkWidget *wifi_list;
-    GtkWidget *wifi_hidden_network;
 
     /* 3G widgets */
     GtkWidget *mobile_label;
@@ -45,21 +73,16 @@ typedef struct {
     gboolean mobile_list_populated;
     gulong mobile_toggled_id;
 
-    GtkWidget *close;
-
     gboolean disposed;
 } NmnNewConnectionPrivate;
 
 GtkWidget *
-nmn_new_connection_create (GtkBuilder *builder,
-                           NmnNMData *nm_data)
+nmn_new_connection_create (NmnModel *model)
 {
-    g_return_val_if_fail (GTK_IS_BUILDER (builder), NULL);
-    g_return_val_if_fail (NMN_IS_NM_DATA (nm_data), NULL);
+    g_return_val_if_fail (NMN_IS_MODEL (model), NULL);
 
     return GTK_WIDGET (g_object_new (NMN_TYPE_NEW_CONNECTION,
-                                     NMN_NEW_CONNECTION_BUILDER, builder,
-                                     NMN_NEW_CONNECTION_NM_DATA, nm_data,
+                                     NMN_NEW_CONNECTION_MODEL, model,
                                      NULL));
 }
 
@@ -75,30 +98,41 @@ connect_cb (gpointer user_data,
             GError *error)
 {
     /* FIXME: Report the error somewhere */
-    g_debug ("connect_cb: %s %s", object_path,
-             error ? error->message : "success");
+    g_warning ("connect_cb: %s %s", object_path,
+               error ? error->message : "success");
+}
+
+static void
+save_timestamp_cb (NMSettingsConnectionInterface *connection,
+		   GError *error,
+		   gpointer user_data)
+{
+	if (error) {
+		g_warning ("Error saving connection %s timestamp: (%d) %s",
+			   nm_connection_get_path (NM_CONNECTION (connection)),
+			   error->code,
+			   error->message);
+	}
 }
 
 static gboolean
-save_connection (NmnNMData *data, NMConnection *connection)
+save_connection (NmnModel *model, NMConnection *connection)
 {
-    NMAGConfConnection *exported;
-    NMAGConfSettings *settings;
+    NMSettingConnection *s_con;
+    NMSettingsConnectionInterface *gconf_connection;
+    NMGConfSettings *settings;
 
-    settings = NMA_GCONF_SETTINGS (nmn_nm_data_get_user_settings (data));
+    settings = NM_GCONF_SETTINGS (nmn_model_get_user_settings (model));
 
-	exported = nma_gconf_settings_get_by_connection (settings, connection);
-	if (exported) {
-		/* Not a new or system connection, save the updated settings to GConf */
-		nma_gconf_connection_save (exported);
-        return TRUE;
-    }
+    gconf_connection = nm_settings_interface_get_connection_by_path (NM_SETTINGS_INTERFACE (settings),
+                                                                     nm_connection_get_path (connection));
+    if (!gconf_connection)
+        return FALSE;
+    s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
+    g_object_set (s_con, NM_SETTING_CONNECTION_TIMESTAMP, (guint64) time (NULL), NULL);
+    nm_settings_connection_interface_update (gconf_connection, save_timestamp_cb, NULL);
 
-    /* FIXME: Try fuzzy match */
-
-    exported = nma_gconf_settings_add_connection (settings, connection);
-
-    return exported != NULL;
+    return TRUE;
 }
 
 static void
@@ -113,9 +147,9 @@ wifi_dialog_done (NMAWirelessDialog *dialog,
         goto done;
 
     connection = nma_wireless_dialog_get_connection (dialog);
-    save_connection (priv->nm_data, connection);
+    save_connection (priv->model, connection);
 
-    nm_client_activate_connection (NM_CLIENT (priv->nm_data),
+    nm_client_activate_connection (nmn_model_get_client (priv->model),
                                    NM_DBUS_SERVICE_USER_SETTINGS,
                                    nm_connection_get_path (connection),
                                    nma_wireless_dialog_get_device (dialog),
@@ -134,26 +168,46 @@ wifi_search (GtkButton *button, gpointer user_data)
     NmnNewConnectionPrivate *priv = GET_PRIVATE (user_data);
     GtkWidget *dialog;
 
-    dialog = nma_wireless_dialog_hidden_new (NM_CLIENT (priv->nm_data));
+    dialog = nma_wireless_dialog_hidden_new (nmn_model_get_client (priv->model));
     g_signal_connect (dialog, "done", G_CALLBACK (wifi_dialog_done), user_data);
     nma_wireless_dialog_show (NMA_WIRELESS_DIALOG (dialog));
 }
 
+static gboolean
+wifi_model_visible_func (GtkTreeModel *model,
+                         GtkTreeIter  *iter,
+                         gpointer      data)
+{
+    NMListItem *item = NULL;
+    gboolean visible = FALSE;
+
+    gtk_tree_model_get (model, iter, NM_LIST_MODEL_COL_ITEM, &item, -1);
+    if (item) {
+        /* Show unconfigured WiFi items */
+        if (NM_IS_WIFI_ITEM (item) && !nm_connection_item_get_connection (NM_CONNECTION_ITEM (item)))
+            visible = TRUE;
+
+        g_object_unref (item);
+    }
+
+    return visible;
+}
+
 static void
 wifi_page_init (NmnNewConnection *connection)
 {
     NmnNewConnectionPrivate *priv = GET_PRIVATE (connection);
-    GtkWidget *container;
+    GtkTreeModel *wifi_model;
 
-    container = GTK_WIDGET (gtk_builder_get_object (priv->builder, "new_wifi_container"));
-    priv->wifi_list = nmn_wifi_list_new (priv->nm_data);
-    g_signal_connect (priv->wifi_list, "connect-requested", G_CALLBACK (connect_requested), connection);
-    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (container), priv->wifi_list);
+    wifi_model = gtk_tree_model_filter_new (GTK_TREE_MODEL (priv->model), NULL);
+    gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (wifi_model),
+                                            wifi_model_visible_func, NULL, NULL);
+
+    priv->wifi_list = nmn_list_new_with_model (wifi_model);
+    g_object_unref (wifi_model);
+    //g_signal_connect (priv->wifi_list, "connect-requested", G_CALLBACK (connect_requested), connection);
+    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (priv->wifi_list_container), priv->wifi_list);
     gtk_widget_show_all (priv->wifi_list);
-
-    priv->wifi_hidden_network = GTK_WIDGET (gtk_builder_get_object (priv->builder, "new_wifi_hidden"));
-    g_signal_connect (priv->wifi_hidden_network, "clicked", G_CALLBACK (wifi_search), connection);
-    g_signal_connect (priv->wifi_hidden_network, "clicked", G_CALLBACK (connect_requested), connection);
 }
 
 /* Mobile */
@@ -163,61 +217,93 @@ wifi_page_init (NmnNewConnection *connection)
 #define MOBILE_COL_METHOD   2
 
 static gboolean
-mobile_have_device (NmnNewConnection *connection, NmnMobileProviderType *type)
+mobile_have_device (NmnNewConnection *connection, NMMobileProviderType *type)
 {
     NmnNewConnectionPrivate *priv = GET_PRIVATE (connection);
+    NMClient *client;
     const GPtrArray *devices;
     int i;
     gboolean found = FALSE;
 
-    devices = nm_client_get_devices (NM_CLIENT (priv->nm_data));
+    client = nmn_model_get_client (priv->model);
+    devices = nm_client_get_devices (client);
     for (i = 0; !found && devices && i < devices->len; i++) {
         NMDevice *device = NM_DEVICE (g_ptr_array_index (devices, i));
 
         if (NM_IS_GSM_DEVICE (device)) {
             found = TRUE;
-            *type = NMN_MOBILE_ACCESS_METHOD_TYPE_GSM;
+            *type = NM_MOBILE_ACCESS_METHOD_TYPE_GSM;
         }
 
         if (NM_IS_CDMA_DEVICE (device)) {
             found = TRUE;
-            *type = NMN_MOBILE_ACCESS_METHOD_TYPE_CDMA;
+            *type = NM_MOBILE_ACCESS_METHOD_TYPE_CDMA;
         }
     }
 
     return found;
 }
 
+static gboolean
+gsm_is_locked (NmnModel *nmn_model)
+{
+    GtkTreeModel *model = GTK_TREE_MODEL (nmn_model);
+    GtkTreeIter iter;
+    gboolean valid;
+    gboolean found = FALSE;
+
+    valid = gtk_tree_model_get_iter_first (model, &iter);
+    while (valid && !found) {
+        NMListItem *item = NULL;
+
+        gtk_tree_model_get (model, &iter, NM_LIST_MODEL_COL_ITEM, &item, -1);
+        if (item) {
+            if (NM_IS_GSM_PIN_REQUEST_ITEM (item))
+                found = TRUE;
+
+            g_object_unref (item);
+        }
+
+        valid = gtk_tree_model_iter_next (model, &iter);
+    }
+
+    return found;
+}
+
 static void
 mobile_status_update (NmnNewConnection *connection)
 {
     NmnNewConnectionPrivate *priv = GET_PRIVATE (connection);
-    NmnMobileProviderType type;
-    gboolean found;
+    NMMobileProviderType type;
+    gboolean active;
 
-    if (!nmn_nm_data_modems_get_active (priv->nm_data)) {
-        found = FALSE;
-        gtk_label_set_text (GTK_LABEL (priv->mobile_label), "3G disabled");
+    if (!nmn_model_modems_get_active (priv->model)) {
+        active = FALSE;
+        gtk_label_set_text (GTK_LABEL (priv->mobile_label), _("3G disabled"));
     } else {
-        found = mobile_have_device (connection, &type);
+        active = mobile_have_device (connection, &type);
 
-        if (found)
-            gtk_label_set_text (GTK_LABEL (priv->mobile_label), "Internal 3G modem and SIM card detected");
-        else
-            gtk_label_set_text (GTK_LABEL (priv->mobile_label), "No modems detected");
+        if (active) {
+            if (type == NM_MOBILE_ACCESS_METHOD_TYPE_GSM && gsm_is_locked (priv->model)) {
+                gtk_label_set_text (GTK_LABEL (priv->mobile_label), _("3G modem is locked"));
+                active = FALSE;
+            } else
+                gtk_label_set_text (GTK_LABEL (priv->mobile_label), _("Internal 3G modem and SIM card detected"));
+        } else
+            gtk_label_set_text (GTK_LABEL (priv->mobile_label), _("No modems detected"));
     }
 
-    gtk_widget_set_sensitive (priv->mobile_list, found);
-    gtk_widget_set_sensitive (priv->mobile_save, found);
+    gtk_widget_set_sensitive (priv->mobile_list, active);
+    gtk_widget_set_sensitive (priv->mobile_save, active);
 }
 
 static gboolean
-method_list_has_type (GSList *list, NmnMobileProviderType type)
+method_list_has_type (GSList *list, NMMobileProviderType type)
 {
     GSList *iter;
 
     for (iter = list; iter; iter = iter->next) {
-        NmnMobileAccessMethod *method = iter->data;
+        NMMobileAccessMethod *method = iter->data;
 
         if (method->type == type)
             return TRUE;
@@ -227,12 +313,12 @@ method_list_has_type (GSList *list, NmnMobileProviderType type)
 }
 
 static gboolean
-provider_list_has_type (GSList *list, NmnMobileProviderType type)
+provider_list_has_type (GSList *list, NMMobileProviderType type)
 {
     GSList *iter;
 
     for (iter = list; iter; iter = iter->next) {
-        NmnMobileProvider *provider = (NmnMobileProvider *) iter->data;
+        NMMobileProvider *provider = (NMMobileProvider *) iter->data;
 
         if (method_list_has_type (provider->methods, type))
             return TRUE;
@@ -243,7 +329,7 @@ provider_list_has_type (GSList *list, NmnMobileProviderType type)
 
 typedef struct {
     GtkTreeStore *store;
-    NmnMobileProviderType provider_type;
+    NMMobileProviderType provider_type;
 
     GtkTreeIter *parent_iter;
 } MobilePopulateInfo;
@@ -251,7 +337,7 @@ typedef struct {
 static void
 add_one_method (gpointer data, gpointer user_data)
 {
-    NmnMobileAccessMethod *method = (NmnMobileAccessMethod *) data;
+    NMMobileAccessMethod *method = (NMMobileAccessMethod *) data;
     MobilePopulateInfo *info = (MobilePopulateInfo *) user_data;
     GtkTreeIter iter;
 
@@ -260,7 +346,7 @@ add_one_method (gpointer data, gpointer user_data)
 
     gtk_tree_store_append (info->store, &iter, info->parent_iter);
 
-    if (method->type == NMN_MOBILE_ACCESS_METHOD_TYPE_GSM) {
+    if (method->type == NM_MOBILE_ACCESS_METHOD_TYPE_GSM) {
         char *txt;
 
         txt = g_strdup_printf ("%s (APN %s)", method->name, method->gsm_apn);
@@ -281,7 +367,7 @@ add_one_method (gpointer data, gpointer user_data)
 static void
 add_one_provider (gpointer data, gpointer user_data)
 {
-    NmnMobileProvider *provider = (NmnMobileProvider *) data;
+    NMMobileProvider *provider = (NMMobileProvider *) data;
     MobilePopulateInfo *info = (MobilePopulateInfo *) user_data;
     GtkTreeIter *country_iter;
     GtkTreeIter iter;
@@ -329,17 +415,17 @@ mobile_list_populate (NmnNewConnection *connection)
     GtkTreeModel *sort_model;
     GtkCellRenderer *renderer;
     GtkTreeViewColumn *column;
-    NmnMobileProviderType provider_type;
+    NMMobileProviderType provider_type;
     MobilePopulateInfo info;
 
     if (priv->mobile_list_populated || !mobile_have_device (connection, &provider_type))
         return;
 
-    mobile_providers = nmn_mobile_providers_parse (NULL);
+    mobile_providers = nm_mobile_providers_parse (NULL);
     if (!mobile_providers)
         return;
 
-    store = gtk_tree_store_new (3, G_TYPE_STRING, NMN_TYPE_MOBILE_PROVIDER, NMN_TYPE_MOBILE_ACCESS_METHOD);
+    store = gtk_tree_store_new (3, G_TYPE_STRING, NM_TYPE_MOBILE_PROVIDER, NM_TYPE_MOBILE_ACCESS_METHOD);
 
     info.store = store;
     info.provider_type = provider_type;
@@ -370,18 +456,20 @@ mobile_tree_selection_changed (GtkTreeSelection *selection,
     NmnNewConnection *connection = (NmnNewConnection *) user_data;
     NmnNewConnectionPrivate *priv = GET_PRIVATE (connection);
     GtkTreeModel *model;
-    NmnMobileAccessMethod *method = NULL;
+    NMMobileAccessMethod *method = NULL;
     GtkTreeIter iter;
 
     if (gtk_tree_selection_get_selected (selection, &model, &iter))
         gtk_tree_model_get (model, &iter, MOBILE_COL_METHOD, &method, -1);
 
     gtk_widget_set_sensitive (priv->mobile_save, method != NULL);
+    if (method)
+        nm_mobile_access_method_unref (method);
 }
 
 static NMConnection *
-mobile_new_connection (NmnMobileProvider *provider,
-                       NmnMobileAccessMethod *method)
+mobile_new_connection (NMMobileProvider *provider,
+                       NMMobileAccessMethod *method)
 {
     NMConnection *connection;
     NMSetting *type_setting;
@@ -391,7 +479,7 @@ mobile_new_connection (NmnMobileProvider *provider,
 
     connection = nm_connection_new ();
 
-    if (method->type == NMN_MOBILE_ACCESS_METHOD_TYPE_GSM) {
+    if (method->type == NM_MOBILE_ACCESS_METHOD_TYPE_GSM) {
         type_setting = nm_setting_gsm_new ();
         g_object_set (type_setting,
                       NM_SETTING_GSM_NUMBER,   "*99#",
@@ -402,7 +490,7 @@ mobile_new_connection (NmnMobileProvider *provider,
 
         /* FIXME: Choose the network_id more intelligently */
         if (g_slist_length (method->gsm_mcc_mnc) == 1) {
-            NmnGsmMccMnc *mcc_mnc = (NmnGsmMccMnc *) method->gsm_mcc_mnc->data;
+            NMGsmMccMnc *mcc_mnc = (NMGsmMccMnc *) method->gsm_mcc_mnc->data;
             char *network_id;
 
             network_id = g_strconcat (mcc_mnc->mcc, mcc_mnc->mnc, NULL);
@@ -412,7 +500,7 @@ mobile_new_connection (NmnMobileProvider *provider,
 
             g_free (network_id);
         }
-    } else if (method->type == NMN_MOBILE_ACCESS_METHOD_TYPE_CDMA) {
+    } else if (method->type == NM_MOBILE_ACCESS_METHOD_TYPE_CDMA) {
         type_setting = nm_setting_cdma_new ();
         g_object_set (type_setting, NM_SETTING_CDMA_NUMBER, "#777", NULL);
     } else
@@ -477,8 +565,8 @@ mobile_save_clicked (GtkButton *button, gpointer user_data)
     NmnNewConnectionPrivate *priv = GET_PRIVATE (self);
     GtkTreeSelection *selection;
     GtkTreeModel *model;
-    NmnMobileProvider *provider = NULL;
-    NmnMobileAccessMethod *method = NULL;
+    NMMobileProvider *provider = NULL;
+    NMMobileAccessMethod *method = NULL;
     GtkTreeIter method_iter;
     GtkTreeIter provider_iter;
 
@@ -497,16 +585,21 @@ mobile_save_clicked (GtkButton *button, gpointer user_data)
 
         connection = NULL;
         connection = mobile_new_connection (provider, method);
-        nmn_mobile_provider_unref (provider);
 
         if (connection) {
-            nma_gconf_settings_add_connection (NMA_GCONF_SETTINGS (nmn_nm_data_get_user_settings (priv->nm_data)),
-                                               connection);
+            nm_gconf_settings_add_connection (NM_GCONF_SETTINGS (nmn_model_get_user_settings (priv->model)),
+                                              connection);
             g_object_unref (connection);
         }
 
         gtk_widget_hide (GTK_WIDGET (self));
     }
+
+    if (provider)
+        nm_mobile_provider_unref (provider);
+
+    if (method)
+        nm_mobile_access_method_unref (method);
 }
 
 static void
@@ -518,7 +611,6 @@ mobile_device_added (NMClient *client,
         NmnNewConnection *connection = NMN_NEW_CONNECTION (user_data);
 
         mobile_list_populate (connection);
-        mobile_status_update (connection);
     }
 }
 
@@ -526,32 +618,105 @@ static void
 mobile_page_init (NmnNewConnection *connection)
 {
     NmnNewConnectionPrivate *priv = GET_PRIVATE (connection);
-    GtkTreeSelection *selection;
+    NMClient *client = nmn_model_get_client (priv->model);
 
-    g_signal_connect (connection, "realize", G_CALLBACK (mobile_list_populate), NULL);
-
-    priv->mobile_label = GTK_WIDGET (gtk_builder_get_object (priv->builder, "new_3g_label"));
-    priv->mobile_list = GTK_WIDGET (gtk_builder_get_object (priv->builder, "new_3g_list"));
-    priv->mobile_save = GTK_WIDGET (gtk_builder_get_object (priv->builder, "new_3g_save"));
-    g_signal_connect (priv->mobile_save, "clicked", G_CALLBACK (mobile_save_clicked), connection);
-
-    g_signal_connect (priv->nm_data, "device-added", G_CALLBACK (mobile_device_added), connection);
-    g_signal_connect_swapped (priv->nm_data, "device-removed", G_CALLBACK (mobile_status_update), connection);
+    g_signal_connect (client, "device-added", G_CALLBACK (mobile_device_added), connection);
+    g_signal_connect_swapped (client, "device-removed", G_CALLBACK (mobile_status_update), connection);
     
-    priv->mobile_toggled_id = g_signal_connect_swapped (priv->nm_data, "modems-toggled",
+    priv->mobile_toggled_id = g_signal_connect_swapped (priv->model, "modems-toggled",
                                                         G_CALLBACK (mobile_status_update),
                                                         connection);
+}
 
-    mobile_status_update (connection);
+void
+nmn_new_connection_update (NmnNewConnection *self)
+{
+    g_return_if_fail (NMN_IS_NEW_CONNECTION (self));
 
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->mobile_list));
-	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
-    g_signal_connect (selection, "changed", G_CALLBACK (mobile_tree_selection_changed), connection);
+    mobile_list_populate (self);
+    mobile_status_update (self);
 }
 
 static void
-nmn_new_connection_init (NmnNewConnection *connection)
+emit_close (NmnNewConnection *self)
 {
+    g_signal_emit (self, signals[CLOSE], 0);
+}
+
+static void
+nmn_new_connection_init (NmnNewConnection *self)
+{
+    NmnNewConnectionPrivate *priv = GET_PRIVATE (self);
+    GtkWidget *w;
+    GtkWidget *notebook;
+    GtkWidget *box;
+    GtkWidget *button;
+    GtkTreeSelection *selection;
+    char *label;
+
+    gtk_box_set_spacing (GTK_BOX (self), 6);
+    gtk_container_set_border_width (GTK_CONTAINER (self), 6);
+
+    label = g_strdup_printf ("<big><b>%s</b></big>", _("Add a new connection"));
+    w = gtk_label_new (NULL);
+    gtk_label_set_markup (GTK_LABEL (w), label);
+    g_free (label);
+    gtk_misc_set_alignment (GTK_MISC (w), 0.0, 0.5);
+    gtk_box_pack_start (GTK_BOX (self), w, FALSE, FALSE, 0);
+
+    notebook = gtk_notebook_new ();
+    gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_LEFT);
+    gtk_box_pack_start (GTK_BOX (self), notebook, TRUE, TRUE, 0);
+
+    /* WiFi page */
+    box = gtk_vbox_new (FALSE, 6);
+    priv->wifi_list_container = gtk_scrolled_window_new (NULL, NULL);
+    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->wifi_list_container),
+                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+    gtk_box_pack_start (GTK_BOX (box), priv->wifi_list_container, TRUE, TRUE, 0);
+
+    w = gtk_alignment_new (0.0, 0.5, 0.0, 1.0);
+    gtk_box_pack_start (GTK_BOX (box), w, FALSE, FALSE, 0);
+    button = gtk_button_new_with_label (_("Hidden network"));
+    gtk_container_add (GTK_CONTAINER (w), button);
+    g_signal_connect (button, "clicked", G_CALLBACK (wifi_search), self);
+    g_signal_connect (button, "clicked", G_CALLBACK (connect_requested), self);
+
+    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), box, gtk_label_new (_("WiFi")));
+
+    /* 3G page */
+    box = gtk_vbox_new (FALSE, 6);
+    priv->mobile_label = gtk_label_new (NULL);
+    gtk_misc_set_alignment (GTK_MISC (priv->mobile_label), 0.0, 0.5);
+    gtk_box_pack_start (GTK_BOX (box), priv->mobile_label, FALSE, TRUE, 0);
+
+    w = gtk_scrolled_window_new (NULL, NULL);
+    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (w), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+    gtk_box_pack_start (GTK_BOX (box), w, TRUE, TRUE, 0);
+
+    priv->mobile_list = gtk_tree_view_new ();
+    gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (priv->mobile_list), FALSE);
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->mobile_list));
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+    g_signal_connect (selection, "changed", G_CALLBACK (mobile_tree_selection_changed), self);
+    gtk_container_add (GTK_CONTAINER (w), priv->mobile_list);
+
+    w = gtk_alignment_new (0.0, 0.5, 0.0, 1.0);
+    gtk_box_pack_start (GTK_BOX (box), w, FALSE, FALSE, 0);
+    priv->mobile_save = gtk_button_new_with_label (_("Save connection"));
+    gtk_container_add (GTK_CONTAINER (w), priv->mobile_save);
+    g_signal_connect (priv->mobile_save, "clicked", G_CALLBACK (mobile_save_clicked), self);
+
+    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), box, gtk_label_new (_("3G")));
+
+    /* Close button */
+    w = gtk_alignment_new (0.0, 0.5, 0.0, 1.0);
+    gtk_box_pack_start (GTK_BOX (self), w, FALSE, FALSE, 0);
+    button = gtk_button_new_with_label (_("Return to Networking"));
+    gtk_container_add (GTK_CONTAINER (w), button);
+    g_signal_connect_swapped (button, "clicked", G_CALLBACK (emit_close), self);
+
+    gtk_widget_show_all (GTK_WIDGET (self));
 }
 
 static GObject*
@@ -561,7 +726,6 @@ constructor (GType type,
 {
     GObject *object;
     NmnNewConnectionPrivate *priv;
-    GtkWidget *container;
 
     object = G_OBJECT_CLASS (nmn_new_connection_parent_class)->constructor
         (type, n_construct_params, construct_params);
@@ -571,21 +735,15 @@ constructor (GType type,
 
     priv = GET_PRIVATE (object);
 
-    if (!priv->builder || !priv->nm_data) {
+    if (!priv->model) {
         g_warning ("Missing constructor arguments");
         g_object_unref (object);
         return NULL;
     }
 
-    container = GTK_WIDGET (gtk_builder_get_object (priv->builder, "new_main_container"));
-    gtk_widget_reparent (container, GTK_WIDGET (object));
-
     wifi_page_init (NMN_NEW_CONNECTION (object));
     mobile_page_init (NMN_NEW_CONNECTION (object));
 
-    priv->close = GTK_WIDGET (gtk_builder_get_object (priv->builder, "new_close"));
-    g_signal_connect_swapped (priv->close, "clicked", G_CALLBACK (gtk_widget_hide), object);
-
     return object;
 }
 
@@ -596,13 +754,9 @@ set_property (GObject *object, guint prop_id,
     NmnNewConnectionPrivate *priv = GET_PRIVATE (object);
 
     switch (prop_id) {
-    case PROP_BUILDER:
+    case PROP_MODEL:
         /* Construct only */
-        priv->builder = g_value_dup_object (value);
-        break;
-    case PROP_NM_DATA:
-        /* Construct only */
-        priv->nm_data = g_value_dup_object (value);
+        priv->model = g_value_dup_object (value);
         break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -617,11 +771,8 @@ get_property (GObject *object, guint prop_id,
     NmnNewConnectionPrivate *priv = GET_PRIVATE (object);
 
     switch (prop_id) {
-    case PROP_BUILDER:
-        g_value_set_object (value, priv->builder);
-        break;
-    case PROP_NM_DATA:
-        g_value_set_object (value, priv->nm_data);
+    case PROP_MODEL:
+        g_value_set_object (value, priv->model);
         break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -639,16 +790,10 @@ dispose (GObject *object)
 
     priv->disposed = TRUE;
 
-    if (priv->wifi_list)
-        gtk_widget_destroy (priv->wifi_list);
+    g_signal_handler_disconnect (priv->model, priv->mobile_toggled_id);
 
-    if (priv->builder)
-        g_object_unref (priv->builder);
-
-    g_signal_handler_disconnect (priv->nm_data, priv->mobile_toggled_id);
-
-    if (priv->nm_data)
-        g_object_unref (priv->nm_data);
+    if (priv->model)
+        g_object_unref (priv->model);
 
     G_OBJECT_CLASS (nmn_new_connection_parent_class)->dispose (object);
 }
@@ -668,18 +813,20 @@ nmn_new_connection_class_init (NmnNewConnectionClass *class)
 
     /* properties */
     g_object_class_install_property
-        (object_class, PROP_BUILDER,
-         g_param_spec_object (NMN_NEW_CONNECTION_BUILDER,
-                              "GtkBuilder",
-                              "GtkBuilder",
-                              GTK_TYPE_BUILDER,
+        (object_class, PROP_MODEL,
+         g_param_spec_object (NMN_NEW_CONNECTION_MODEL,
+                              "NmnModel",
+                              "NmnModel",
+                              NMN_TYPE_MODEL,
                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
-    g_object_class_install_property
-        (object_class, PROP_NM_DATA,
-         g_param_spec_object (NMN_NEW_CONNECTION_NM_DATA,
-                              "NmnNMData",
-                              "NmnNMData",
-                              NMN_TYPE_NM_DATA,
-                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+    /* signals */
+    signals[CLOSE] = g_signal_new 
+        ("close",
+         G_OBJECT_CLASS_TYPE (class),
+         G_SIGNAL_RUN_LAST,
+         G_STRUCT_OFFSET (NmnNewConnectionClass, close),
+         NULL, NULL,
+         g_cclosure_marshal_VOID__VOID,
+         G_TYPE_NONE, 0);
 }
